;*****************************************************
; Sample source program 'Calc_DOW.asm'               *
; Copyright (C) 2014. COHT Inc. All Rights Reserved. *
;*****************************************************
;
; predefined handle value
;
STD_IN  equ     0        ; std_in handle
STD_OUT equ     1        ; std_out handle
STD_ERR equ     2        ; std_err handle
;
; Program Segment Prefix
;
ARG_LEN equ     80h      ; command parameter length
ARG_BUF equ     81h      ; command parameters
;
        ORG     0100H
Entry:
        call    ArgSplit ; split to argument
        mov     al,[ARGC]; get argument count
        cmp     al,3
        jge     $+5
        jmp     BadArg   ; argument error
;
        mov     bx,ARGV
        mov     si,[bx]  ; first argument
        add     bx,2
        call    StrToInt ; get value
        mov     [YEAR],ax
;
        mov     si,[bx]  ; second argument
        add     bx,2
        call    StrToInt ; get value
        cmp     ax,2
        ja      NoAdjust
;
        sub     word ptr YEAR,1;previous year
        add     ax,12    ; 13th-14th month
NoAdjust:
        sub     ax,2
        mov     [MONTH],ax
;
        mov     si,[bx]  ; third argument
        call    StrToInt ; get value
        mov     [DAY],ax
;
        mov     ax,[YEAR]; 365 * Y
        cwd
        mov     cx,365
        xor     bx,bx
        call    DWordMul
        mov     [JULIUSL],ax
        mov     [JULIUSH],dx
;
        mov     ax,[YEAR]; Y / 4
        cwd
        mov     cx,4
        xor     bx,bx
        call    DWordDiv
        add     [JULIUSL],ax
        adc     [JULIUSH],dx
;
        mov     ax,[YEAR]; Y / 100
        cwd
        mov     cx,100
        xor     bx,bx
        call    DWordDiv
        sub     [JULIUSL],ax
        sbb     [JULIUSH],dx
;
        mov     ax,[YEAR]; Y / 400
        cwd
        mov     cx,400
        xor     bx,bx
        call    DWordDiv
        add     [JULIUSL],ax
        adc     [JULIUSH],dx
;
        mov     ax,[MONTH]; M * 30
        cwd
        mov     cx,30
        xor     bx,bx
        call    DWordMul
        add     [JULIUSL],ax
        adc     [JULIUSH],dx
;
        mov     ax,[MONTH]; M * 59 / 100
        cwd
        mov     cx,59
        xor     bx,bx
        call    DWordMul
        mov     cx,100
        xor     bx,bx
        call    DWordDiv
        add     ax,[JULIUSL]
        adc     dx,[JULIUSH]
;
        add     ax,[DAY] ; + D
        adc     dx,0
        sub     ax,5C00H ; - 678912
        sbb     dx,000AH
        mov     cx,7     ; / 7
        xor     bx,bx
        call    DWordDiv
;
        mov     bx,DOWStr; Day Of the Week String
        add     cx,cx    ; rest * 2
        add     bx,cx    ; translated pointer
        mov     ax,[bx]  ; get string address
        mov     dx,ax    ; save for DOS call
        call    StrLen   ; get string length in CX
;
        mov     bx,STD_OUT
        mov     ah,40h   ;  write to file handle function
        int     21h
;
        mov     dx,CRLF  ; CR/LF
        mov     cx,2
        mov     bx,STD_OUT
        mov     ah,40h   ;  write to file handle function
        int     21h
;
        mov     al,0     ; return code
        mov     ah,4ch   ; end program function
        int     21h
;
BadArg:
        mov     dx,PARAERR;
        mov     cx,19
        mov     bx,STD_OUT
        mov     ah,40h   ;  write to file handle function
        int     21h
;
        mov     al,1     ; return code
        mov     ah,4ch   ; end program function
        int     21h
;
YEAR:   dw      0
MONTH:  dw      0
DAY:    dw      0
JULIUSL:dw      0
JULIUSH:dw      0
;
PARAERR:db      "Parameter Error !"
CRLF:   db      13,10
;
DOWStr: dw      WED
        dw      THR
        dw      FRI
        dw      SAT
        dw      SUN
        dw      MON
        dw      TUE
;
SUN:    db      "Sunday",0
MON:    db      "Monday",0
TUE:    db      "Tuesday",0
WED:    db      "Wednesday",0
THR:    db      "Thursday",0
FRI:    db      "Friday",0
SAT:    db      "Saturday",0
;
; subroutine ArgSplit
; byte [ARGC]= OUT:argument count
; word [ARGV]= OUT:argument string array
;
ArgSplit equ    $
        xor     ax,ax
        mov     [ARGC],al; clear ARGC
        mov     bx,ARGV  ; top of ARGV array
;
        mov     cl,[ARG_LEN]; get command line string length
        xor     ch,ch    ; clear upper byte
        mov     si,ARG_BUF; get command line pointer
        cld              ; increase direction
;
ArgGet:
        jcxz    SplitRet ; jump if end of command line
        lodsb            ; get next letter
        dec     cx       ; decrement counter
        cmp     al,20h   ; printable letter?
        jle     ArgGet   ; jump if not
;
        mov     al,[ARGC]; increment ARGC
        inc     al
        mov     [ARGC],al
        mov     ax,si    ; get parameter address in AX
        dec     ax
        mov     [bx],ax  ; store to ARGV
        add     bx,2     ; ARGV++
;
ParaLoop:
        jcxz    SplitRet ; jump if end of command line
        lodsb            ; get next letter
        dec     cx       ; decrement counter
        cmp     al,20h   ; printable letter?
        jle     ArgGet   ; jump if not
        jmp     ParaLoop ; stay in parameter loop
;
SplitRet:
        ret
;
ARGC:   db      0        ; argument count
ARGV:   dw      0        ; argument vector array
        ds      124      ; 2 * 62byte
;
; subroutine StrToInt
; si= IN:string address
; ax= OUT:Int value
;
StrToInt equ    $
        push    dx       ; save regs
        push    bx
        mov     bx,10    ; multiplier
        xor     ax,ax    ; clear VALUE
        mov     dx,ax
;
StrInt50:
        mov     dl,[si]
        inc     si
        cmp     dl,20h   ; terminator?
        jle     StrIntRet; branch if so
;
        mul     bl       ; multiply VALUE with 10
        sub     dl,30h   ; get hex value
        add     ax,dx    ; add as least significant digit
        jmp     StrInt50
;
StrIntRet:
        pop     bx
        pop     dx
        ret
;
; Subroutine DwordAdd
; DX:AX = IN:DWORD value 1
; BX:CX = IN:DWORD value 2
; DX:AX = OUT:DWORD result(DX:AX + BX:CX)
;
DwordAdd equ    $
        add     ax,cx    ; add as dword
        adc     dx,bx
        ret
;
; Subroutine DwordSub
; DX:AX = IN:DWORD value 1
; BX:CX = IN:DWORD value 2
; DX:AX = OUT:DWORD result(DX:AX - BX:CX)
;
DwordSub equ    $
        sub     ax,cx    ; subtract as dword
        sbb     dx,bx
        ret
;
;
; Subroutine DwordMul
; DX:AX = IN:DWORD value 1
; BX:CX = IN:DWORD value 2
; DX:AX = OUT:DWORD result(DX:AX * BX:CX)
;
DwordMul equ    $
        push    si       ; save regs
        push    di
        mov     si,ax
        mov     di,dx
        xor     ax,ax    ; initialize with zero
        mov     dx,ax
;
DWMul10:
        or      cx,cx    ; calc end?
        jnz     DWMul20  ; branch if not
        or      bx,bx    ; calc end?
        jz      DWMulRet ; branch if so
;
DWMul20:
        shr     bx,1     ; get least significant bit in carry
        rcr     cx,1
        jnc     DWMul50  ; skip if zero
;
        add     ax,si; add first value to result
        adc     dx,di
DWMul50:
        sal     si,1     ; left shift first value
        rcl     di,1
        jmp     DWMul10
;
DWMulRet:
        pop     di       ; restore regs
        pop     si
        ret
;
; Subroutine DwordDiv
; DX:AX = IN:DWORD value 1
; BX:CX = IN:DWORD value 2
; DX:AX = OUT:DWORD result(DX:AX / BX:CX)
; BX:CX = OUT:DWORD rest
;
DwordDiv equ    $
        push    si       ; save regs
        push    di
        mov     si,1     ; bit pointer
        xor     di,di
        mov     [DIVWK1],di; initialize with zero
        mov     [DIVWK2],di
;
        or      bx,bx    ; 0 div check
        jnz     DWDiv10
        or      cx,cx
        jz      DWDivRet ; 0 Divide error
;
        xchg    bx,cx    ; shift left 16bits
        xchg    di,si
        cmp     dx,bx    ; comp divident with divider
        jc      DWDiv70  ; branch if less
;
        or      bh,bh    ; check b24-b31
        jnz     DWDiv10  ; branch not zero
;
        xchg    bh,bl    ; shift left 24bits
        mov     di,0100h
;
; shift left until divider is greater than divident
;
DWDiv10:
        cmp     dx,bx    ; comp divident with divider
        jc      DWDiv70  ; branch if less
        jnz     DWDiv20
        cmp     ax,cx
        jc      DWDiv70  ; branch if less
;
DWDiv20:
        sal     si,1     ; left shift bit pointer
        rcl     di,1
        sal     cx,1     ; left shift divider
        rcl     bx,1
        jmp     DWDiv10
;
DWDiv30:
        cmp     dx,bx    ; comp divident with divider
        jc      DWDiv70  ; branch if less
        jnz     DWDiv50
        cmp     ax,cx
        jc      DWDiv70  ; branch if less
DWDiv50:
        sub     ax,cx    ; subtract divider
        sbb     dx,bx
        or      [DIVWK1],si; add to result
        or      [DIVWK2],di
DWDiv70:
        shr     bx,1     ; right shift divider
        rcr     cx,1
        shr     di,1     ; right shift bit pointer
        rcr     si,1
        jnc     DWDiv30  ; loop while bit pointer exist
;
DWDivRet:
        mov     cx,ax
        mov     bx,dx
        mov     ax,[DIVWK1]; get result
        mov     dx,[DIVWK2]
        pop     di       ; restore regs
        pop     si
        ret
;
DIVWK1: dw      0
DIVWK2: dw      0
;
; subroutine StrLen
; AX = IN:String pointer
; CX = OUT:String length
;
StrLen  equ     $
        pushf            ; save regs
        push    ax
        push    si
        xor     cx,cx    ; clear counter
        mov     si,ax    ; SI is source string
        cld              ; increase direction
;
PrintChr:
        lodsb            ; get one character
        cmp     al,20h   ; comp with space code(separator)
        jle     SpaceChr ; jump if less or equal
;
        inc     cx       ; increment count
        jmp     PrintChr ; loop
;
SpaceChr:
        pop     si       ; restore regs
        pop     ax
        popf
        ret
;
        end
