;*****************************************************
; Sample source program 'DigClock.asm'               *
; Copyright (C) 2014. COHT Inc. All Rights Reserved. *
;*****************************************************
        ORG     0100H
Entry:
        mov     ah,2Ch   ; get current time
        int     21h
        cmp     [CurrHHMM],cx; Hour/Minutes advanced?
        jnz     UpdatClk ; bra if so
;
        cmp     [CurrSec],dh; Second advanced?
        jz      Entry    ; bra if not
;
UpdatClk:
        mov     [CurrHHMM],cx
        mov     [CurrSec],dh
;
        test    [CursorUp],0ffh; cursor need to up
        jz      StayCrsr ; bra if not
;
        mov     ah,3     ; get cursor position
        mov     bh,0     ; page#
        int     10h
        sub     dh,3     ; 3 lines above
        mov     ah,2     ; set cursor position
        mov     bh,0     ; page#
        int     10h
        mov     cx,[CurrHHMM]
        mov     dh,[CurrSec]
;
StayCrsr:
        mov     di,OUTPORT; output port buffer
        mov     bx,_7SegVal; decode table
        mov     al,ch    ; Hour value
        cbw
        mov     dl,10
        div     dl
;
        xlat
        stosb
        mov     al,ah
        xlat
        stosb
;
        mov     al,cl    ; Minutes value
        cbw
        mov     dl,10
        div     dl
;
        xlat
        stosb
        mov     al,ah
        xlat
        stosb
;
        mov     al,dh    ; Minutes value
        cbw
        mov     dl,10
        div     dl
;
        xlat
        stosb
        mov     al,ah
        xlat
        stosb
;
        call    ClockDsp
        mov     byte ptr CursorUp,1
;
        jmp     Entry
;
        mov     al,0     ; return code
        mov     ah,4ch   ; end program function
        int     21h
;
CurrHHMM:dw     0
CurrSec:db      0
CursorUp:db     0
;
; 7 Segment display decode value(0-9)
;
_7SegVal:db     3Fh,06h,5Bh,4Fh,66h,6Dh,7Dh,07h,7Fh,6Fh
;
; Clock display port buffer(7 segment display port)
;
OUTPORT:db      06h,5Bh,4Fh,66h,6Dh,7Dh
;
; subroutine ClockDsp
; byte*4 [OUTPORT]= IN:7 segment display data (a=b0~g=b6)
;
ClockDsp equ    $
        pushf
        push    ax
        push    cx
        push    dx
        push    si
        cld
;
; 7 seg TOP line
;
        mov     si,OUTPORT
        mov     cx,3
_7SegTop:
        lodsb
        call    Top7Seg
        lodsb
        call    Top7Seg
;
        cmp     cx,1
        jz      TopEnd
;
        mov     ah,9
        mov     dx,BLANK
        int     21h
        loop    _7SegTop
;
TopEnd: mov     ah,9
        mov     dx,CRLF
        int     21h
;
; 7 seg MIDDLE line
;
        mov     si,OUTPORT
        mov     cx,3
_7SegMid:
        lodsb
        call    Mid7Seg
        lodsb
        call    Mid7Seg
;
        cmp     cx,1
        jz      MidEnd
;
        mov     ah,9
        mov     dx,ASTER
        int     21h
        loop    _7SegMid
;
MidEnd: mov     ah,9
        mov     dx,CRLF
        int     21h
;
; 7 seg BOTTOM line
;
        mov     si,OUTPORT
        mov     cx,3
_7SegBot:
        lodsb
        call    Bot7Seg
        lodsb
        call    Bot7Seg
;
        cmp     cx,1
        jz      BotEnd
;
        mov     ah,9
        mov     dx,ASTER
        int     21h
        loop    _7SegBot
;
BotEnd: mov     ah,9
        mov     dx,CRLF
        int     21h
;
        pop     si
        pop     dx
        pop     cx
        pop     ax
        popf
        ret
;
BLANK:  db      "  $"
ASTER:  db      " *$"
CRLF:   db      13,10,'$'
;
; subroutine Top7Seg
; AL= IN:7 segment display data (a=b0~g=b6)
;
Top7Seg equ     $
        push    ax
        push    dx
        mov     dh,al
;
        mov     dl,20h   ; left mergine
        mov     ah,2
        int     21h
;
        mov     dl,20h   ; left blank
        mov     ah,2
        int     21h
;
        mov     dl,5Fh   ; segment 'a'
        test    dh,01h
        jnz     SegAOn
;
        mov     dl,20h   ; segment is off
SegAOn: mov     ah,2
        int     21h
;
        mov     dl,20h   ; right blank
        mov     ah,2
        int     21h
;
        pop     dx
        pop     ax
        ret
;
; subroutine Mid7Seg
; AL= IN:7 segment display data (a=b0~g=b6)
;
Mid7Seg equ     $
        push    ax
        push    dx
        mov     dh,al
;
        mov     dl,20h   ; left mergine
        mov     ah,2
        int     21h
;
        mov     dl,7Ch   ; segment 'f'
        test    dh,20h
        jnz     SegFOn
;
        mov     dl,20h   ; segment is off
SegFOn: mov     ah,2
        int     21h
;
        mov     dl,5Fh   ; segment 'g'
        test    dh,40h
        jnz     SegGOn
;
        mov     dl,20h   ; segment is off
SegGOn: mov     ah,2
        int     21h
;
        mov     dl,7Ch   ; segment 'b'
        test    dh,02h
        jnz     SegBOn
;
        mov     dl,20h   ; segment is off
SegBOn: mov     ah,2
        int     21h
;
        pop     dx
        pop     ax
        ret
;
; subroutine Bot7Seg
; AL= IN:7 segment display data (a=b0~g=b6)
;
Bot7Seg equ     $
        push    ax
        push    dx
        mov     dh,al
;
        mov     dl,20h   ; left mergine
        mov     ah,2
        int     21h
;
        mov     dl,7Ch   ; segment 'e'
        test    dh,10h
        jnz     SegEOn
;
        mov     dl,20h   ; segment is off
SegEOn: mov     ah,2
        int     21h
;
        mov     dl,5Fh   ; segment 'd'
        test    dh,08h
        jnz     SegDOn
;
        mov     dl,20h   ; segment is off
SegDOn: mov     ah,2
        int     21h
;
        mov     dl,7Ch   ; segment 'c'
        test    dh,04h
        jnz     SegCOn
;
        mov     dl,20h   ; segment is off
SegCOn: mov     ah,2
        int     21h
;
        pop     dx
        pop     ax
        ret
;
DEGITAL:db      "  _ ", "  _ ", '  ', "  _ ", "  _ ",13,10
        db      " |_|", " |_|", ' *', " |_|", " |_|",13,10
        db      " |_|", " |_|", ' *', " |_|", " |_|",13,10
        db      13,10
        db      "    ", "  _ ", "  _ ", "    ", "  _ ", "  _ ", "  _ ", "  _ ", "  _ ", "  _ ",13,10
        db      "   |", "  _|", "  _|", " |_|", " |_ ", " |_ ", "   |", " |_|", " |_|", " | |",13,10
        db      "   |", " |_ ", "  _|", "   |", "  _|", " |_|", "   |", " |_|", "  _|", " |_|",13,10
        db      13,10,'$'
        end
